@@ -48,12 +48,12 @@
  * Process an debug node
  */
 void 
-xsltDebug(xsltTransformContextPtr ctxt ATTRIBUTE_UNUSED, xmlNodePtr node ATTRIBUTE_UNUSED,
+xsltDebug(xsltTransformContextPtr ctxt, xmlNodePtr node ATTRIBUTE_UNUSED,
 	  xmlNodePtr inst ATTRIBUTE_UNUSED, xsltStylePreCompPtr comp ATTRIBUTE_UNUSED) {
     int i, j;
 
     fprintf(stdout, "Templates:\n");
-    for (i = 0, j = ctxt->templNr - 1;((i < 5) && (j >= 0));i++,j--) {
+    for (i = 0, j = ctxt->templNr - 1;((i < 15) && (j >= 0));i++,j--) {
 	fprintf(stdout, "#%d ", i);
 	if (ctxt->templTab[j]->name != NULL)
 	    fprintf(stdout, "name %s ", ctxt->templTab[j]->name);
@@ -64,7 +64,7 @@ xsltDebug(xsltTransformContextPtr ctxt ATTRIBUTE_UNUSED, xmlNodePtr node ATTRIBU
 	fprintf(stdout, "\n");
     }
     fprintf(stdout, "Variables:\n");
-    for (i = 0, j = ctxt->varsNr - 1;((i < 10) && (j >= 0));i++,j--) {
+    for (i = 0, j = ctxt->varsNr - 1;((i < 15) && (j >= 0));i++,j--) {
 	xsltStackElemPtr cur;
 
 	if (ctxt->varsTab[j] == NULL)
@@ -72,12 +72,12 @@ xsltDebug(xsltTransformContextPtr ctxt ATTRIBUTE_UNUSED, xmlNodePtr node ATTRIBU
 	fprintf(stdout, "#%d\n", i);
 	cur = ctxt->varsTab[j];
 	while (cur != NULL) {
-	    if (cur->type == XSLT_ELEM_VARIABLE)
-		fprintf(stdout, "var ");
-	    else if (cur->type == XSLT_ELEM_PARAM)
-		fprintf(stdout, "param ");
-	    else {
+	    if (cur->comp == NULL) {
 		fprintf(stdout, "corrupted !!!\n");
+	    } else if (cur->comp->type == XSLT_FUNC_PARAM) {
+		fprintf(stdout, "param ");
+	    } else if (cur->comp->type == XSLT_FUNC_VARIABLE) {
+		fprintf(stdout, "var ");
 	    }
 	    if (cur->name != NULL)
 		fprintf(stdout, "%s ", cur->name);
