@@ -20,14 +20,17 @@ int		xsltAddKey		(xsltStylesheetPtr style,
 					 const xmlChar *name,
 					 const xmlChar *nameURI,
 					 const xmlChar *match,
-					 const xmlChar *use);
+					 const xmlChar *use,
+					 xmlNodePtr inst);
 xmlNodeSetPtr	xsltGetKey		(xsltTransformContextPtr ctxt,
 					 const xmlChar *name,
 					 const xmlChar *nameURI,
 					 const xmlChar *value);
-void		xsltInitCtxtKeys	(xsltTransformContextPtr ctxt);
+void		xsltInitCtxtKeys	(xsltTransformContextPtr ctxt,
+					 xsltDocumentPtr doc);
 void		xsltFreeKeys		(xsltStylesheetPtr style);
-void		xsltFreeCtxtKeys	(xsltTransformContextPtr ctxt);
+void		xsltFreeDocumentKeys	(xsltDocumentPtr doc);
+
 #ifdef __cplusplus
 }
 #endif
