@@ -93,6 +93,14 @@ xmlChar *
 xsltEvalXPathString(xsltTransformContextPtr ctxt, xmlXPathCompExprPtr comp) {
     xmlChar *ret = NULL;
     xmlXPathObjectPtr res;
+    xmlNodePtr oldInst;
+    xmlNodePtr oldNode;
+    int	oldPos, oldSize;
+
+    oldInst = ctxt->inst;
+    oldNode = ctxt->node;
+    oldPos = ctxt->xpathCtxt->proximityPosition;
+    oldSize = ctxt->xpathCtxt->contextSize;
 
     ctxt->xpathCtxt->node = ctxt->node;
     /* TODO: do we need to propagate the namespaces here ? */
@@ -115,6 +123,10 @@ xsltEvalXPathString(xsltTransformContextPtr ctxt, xmlXPathCompExprPtr comp) {
     xsltGenericDebug(xsltGenericDebugContext,
 	 "xsltEvalXPathString: returns %s\n", ret);
 #endif
+    ctxt->inst = oldInst;
+    ctxt->node = oldNode;
+    ctxt->xpathCtxt->contextSize = oldSize;
+    ctxt->xpathCtxt->proximityPosition = oldPos;
     return(ret);
 }
 
@@ -192,7 +204,10 @@ xsltAttrTemplateValueProcess(xsltTransformContextPtr ctxt, const xmlChar *str) {
 	    expr = xmlStrndup(str, cur - str);
 	    if (expr == NULL)
 		return(ret);
-	    else {
+	    else if (*expr == '{') {
+		ret = xmlStrcat(ret, expr);
+		xmlFree(expr);
+	    } else {
 		xmlXPathCompExprPtr comp;
 		/*
 		 * TODO: keep precompiled form around
@@ -312,6 +327,7 @@ xsltEvalStaticAttrValueTemplate(xsltStylesheetPtr style, xmlNodePtr node,
 xmlAttrPtr
 xsltAttrTemplateProcess(xsltTransformContextPtr ctxt, xmlNodePtr target,
 	                xmlAttrPtr cur) {
+    xmlNsPtr ns;
     xmlAttrPtr ret;
     if ((ctxt == NULL) || (cur == NULL))
 	return(NULL);
@@ -332,14 +348,10 @@ xsltAttrTemplateProcess(xsltTransformContextPtr ctxt, xmlNodePtr target,
 	}
 	return(NULL);
     }
-    ret = xmlNewDocProp(ctxt->output, cur->name, NULL);
-    if (ret == NULL) return(NULL);
-    ret->parent = target;
-    
     if (cur->ns != NULL)
-	ret->ns = xsltGetNamespace(ctxt, cur->parent, cur->ns, target);
+	ns = xsltGetNamespace(ctxt, cur->parent, cur->ns, target);
     else
-	ret->ns = NULL;
+	ns = NULL;
 
     if (cur->children != NULL) {
 	xmlChar *in = xmlNodeListGetString(ctxt->document->doc,
@@ -348,19 +360,16 @@ xsltAttrTemplateProcess(xsltTransformContextPtr ctxt, xmlNodePtr target,
 
 	/* TODO: optimize if no template value was detected */
 	if (in != NULL) {
-	    xmlNodePtr child;
-
             out = xsltAttrTemplateValueProcess(ctxt, in);
-	    child = xmlNewDocText(ctxt->output, out);
-	    xmlAddChild((xmlNodePtr) ret, child);
+	    ret = xmlSetNsProp(target, ns, cur->name, out);
 	    if (out != NULL)
 		xmlFree(out);
 	    xmlFree(in);
 	} else
-	    ret->children = NULL;
+	    ret = xmlSetNsProp(target, ns, cur->name, (const xmlChar *)"");
        
     } else 
-	ret->children = NULL;
+	ret = xmlSetNsProp(target, ns, cur->name, (const xmlChar *)"");
     return(ret);
 }
 
@@ -379,7 +388,7 @@ xmlAttrPtr
 xsltAttrListTemplateProcess(xsltTransformContextPtr ctxt, 
 	                    xmlNodePtr target, xmlAttrPtr cur) {
     xmlAttrPtr ret = NULL;
-    xmlAttrPtr p = NULL,q;
+    xmlAttrPtr q;
     xmlNodePtr oldInsert;
 
     oldInsert = ctxt->insert;
@@ -389,12 +398,8 @@ xsltAttrListTemplateProcess(xsltTransformContextPtr ctxt,
 	if (q != NULL) {
 	    q->parent = target;
 	    q->doc = ctxt->output;
-	    if (p == NULL) {
-		ret = p = q;
-	    } else {
-		p->next = q;
-		q->prev = p;
-		p = q;
+	    if (ret == NULL) {
+		ret = q;
 	    }
 	}
 	cur = cur->next;
