@@ -222,8 +222,10 @@ xsltComputeSortResult(xsltTransformContextPtr ctxt, xmlNodePtr sort) {
     xmlXPathObjectPtr res;
     int len = 0;
     int i;
-    xmlNodePtr oldNode;
     xsltStylePreCompPtr comp;
+    xmlNodePtr oldNode;
+    xmlNodePtr oldInst;
+    int	oldPos, oldSize ;
 
     comp = sort->_private;
     if (comp == NULL) {
@@ -232,14 +234,8 @@ xsltComputeSortResult(xsltTransformContextPtr ctxt, xmlNodePtr sort) {
 	return(NULL);
     }
 
-    if (comp->select == NULL)
+    if ((comp->select == NULL) || (comp->comp == NULL))
 	return(NULL);
-    if (comp->comp == NULL) {
-	comp->comp = xmlXPathCompile(comp->select);
-	if (comp->comp == NULL)
-	    return(NULL);
-    }
-
 
     list = ctxt->nodeList;
     if ((list == NULL) || (list->nodeNr <= 1))
@@ -259,7 +255,11 @@ xsltComputeSortResult(xsltTransformContextPtr ctxt, xmlNodePtr sort) {
     }
 
     oldNode = ctxt->node;
+    oldInst = ctxt->inst;
+    oldPos = ctxt->xpathCtxt->proximityPosition;
+    oldSize = ctxt->xpathCtxt->contextSize;
     for (i = 0;i < len;i++) {
+	ctxt->inst = sort;
 	ctxt->xpathCtxt->contextSize = len;
 	ctxt->xpathCtxt->proximityPosition = i + 1;
 	ctxt->node = list->nodeTab[i];
@@ -294,9 +294,14 @@ xsltComputeSortResult(xsltTransformContextPtr ctxt, xmlNodePtr sort) {
 		    results[i] = NULL;
 		}
 	    }
+	} else {
+	    results[i] = NULL;
 	}
     }
     ctxt->node = oldNode;
+    ctxt->inst = oldInst;
+    ctxt->xpathCtxt->contextSize = oldSize;
+    ctxt->xpathCtxt->proximityPosition = oldPos;
 
     return(results);
 }
@@ -341,6 +346,8 @@ xsltDoSortFunction(xsltTransformContextPtr ctxt, xmlNodePtr *sorts,
     len = list->nodeNr;
 
     resultsTab[0] = xsltComputeSortResult(ctxt, sorts[0]);
+    for (i = 1;i < XSLT_MAX_SORT;i++)
+	resultsTab[i] = NULL;
 
     results = resultsTab[0];
 
@@ -371,8 +378,6 @@ xsltDoSortFunction(xsltTransformContextPtr ctxt, xmlNodePtr *sorts,
 			tst = xmlStrcmp(results[j]->stringval,
 				     results[j + incr]->stringval); 
 		    }
-		    if (tst == 0)
-			tst = results[j]->index > results[j + incr]->index;
 		    if (descending)
 			tst = -tst;
 		}
@@ -414,8 +419,6 @@ xsltDoSortFunction(xsltTransformContextPtr ctxt, xmlNodePtr *sorts,
 				tst = xmlStrcmp(res[j]->stringval,
 					     res[j + incr]->stringval); 
 			    }
-			    if (tst == 0)
-				tst = res[j]->index > res[j + incr]->index;
 			    if (desc)
 				tst = -tst;
 			}
@@ -429,6 +432,9 @@ xsltDoSortFunction(xsltTransformContextPtr ctxt, xmlNodePtr *sorts,
 			depth++;
 		    }
 		}
+		if (tst == 0) {
+		    tst = results[j]->index > results[j + incr]->index;
+		}
 		if (tst > 0) {
 		    tmp = results[j];
 		    results[j] = results[j + incr];
@@ -436,6 +442,18 @@ xsltDoSortFunction(xsltTransformContextPtr ctxt, xmlNodePtr *sorts,
 		    node = list->nodeTab[j];
 		    list->nodeTab[j] = list->nodeTab[j + incr];
 		    list->nodeTab[j + incr] = node;
+		    depth = 1;
+		    while (depth < nbsorts) {
+			if (sorts[depth] == NULL)
+			    break;
+			if (resultsTab[depth] == NULL)
+			    break;
+			res = resultsTab[depth];
+			tmp = res[j];
+			res[j] = res[j + incr];
+			res[j + incr] = tmp;
+			depth++;
+		    }
 		    j -= incr;
 		} else
 		    break;
@@ -488,12 +506,14 @@ xsltSaveResultTo(xmlOutputBufferPtr buf, xmlDocPtr result,
         return(-1);
     }
 
-    /* TODO: when outputing and having imported stylesheets, apply cascade */
     base = buf->written;
 
     XSLT_GET_IMPORT_PTR(method, style, method)
     XSLT_GET_IMPORT_PTR(encoding, style, encoding)
 
+    if ((method == NULL) && (result->type == XML_HTML_DOCUMENT_NODE))
+	method = (const xmlChar *) "html";
+
     if (method == NULL)
 	root = xmlDocGetRootElement(result);
     else
@@ -530,15 +550,11 @@ xsltSaveResultTo(xmlOutputBufferPtr buf, xmlDocPtr result,
 	int standalone;
 	int indent;
 	const xmlChar *version;
-	const xmlChar *doctypePublic;
-	const xmlChar *doctypeSystem;
 
 	XSLT_GET_IMPORT_INT(omitXmlDecl, style, omitXmlDeclaration);
 	XSLT_GET_IMPORT_INT(standalone, style, standalone);
 	XSLT_GET_IMPORT_INT(indent, style, indent);
 	XSLT_GET_IMPORT_PTR(version, style, version)
-	XSLT_GET_IMPORT_PTR(doctypePublic, style, doctypePublic)
-	XSLT_GET_IMPORT_PTR(doctypeSystem, style, doctypeSystem)
 
 	if (omitXmlDecl != 1) {
 	    xmlOutputBufferWriteString(buf, "<?xml version=");
@@ -570,39 +586,6 @@ xsltSaveResultTo(xmlOutputBufferPtr buf, xmlDocPtr result,
 	    }
 	    xmlOutputBufferWriteString(buf, "?>\n");
 	}
-	if ((doctypePublic != NULL) || (doctypeSystem != NULL)) {
-	    xmlNodePtr cur = result->children;
-
-	    while (cur != NULL) {
-		if (cur->type == XML_ELEMENT_NODE)
-		    break;
-		cur = cur->next;
-	    }
-	    if ((cur != NULL) && (cur->name != NULL)) {
-		xmlOutputBufferWriteString(buf, "<!DOCTYPE ");
-		xmlOutputBufferWriteString(buf, (const char *) cur->name);
-		if (doctypePublic != NULL) {
-		    if (doctypeSystem != NULL) {
-			xmlOutputBufferWriteString(buf, " PUBLIC ");
-			xmlBufferWriteQuotedString(buf->buffer,
-				         doctypePublic);
-			xmlOutputBufferWriteString(buf, " ");
-			xmlBufferWriteQuotedString(buf->buffer,
-				         doctypeSystem);
-		    } else {
-			xmlOutputBufferWriteString(buf, " PUBLIC \"-\" ");
-			xmlBufferWriteQuotedString(buf->buffer,
-				         doctypeSystem);
-		    }
-
-		} else {
-		    xmlOutputBufferWriteString(buf, " SYSTEM ");
-		    xmlBufferWriteQuotedString(buf->buffer,
-				     doctypeSystem);
-		}
-		xmlOutputBufferWriteString(buf, ">\n");
-	    }
-	}
 	if (result->children != NULL) {
 	    xmlNodePtr child = result->children;
 
